#ifndef __BDILNK_H__
#define __BDILNK_H__
/*************************************************************************
|  COPYRIGHT (c) 1997 BY ABATRON AG
|*************************************************************************
|
|  PROJECT NAME: BDI Communication Driver
|  FILENAME    : bdilnkmt.h
|
|  COMPILER    : GCC
|
|  TARGET OS   : Linux
|  TARGET HW   : PC
|
|  PROGRAMMER  : Abatron / RD
|  CREATION    : 21.07.2005
|
|*************************************************************************
|
|  DESCRIPTION :
|  DLL for communication with BDI
|
|*************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/*************************************************************************
|  DEFINES
|*************************************************************************/

#define LOAD_FRAME_COUNT        4
#define LOAD_MAX_BLOCK_SIZE     1024


/*************************************************************************
|  TYPEDEFS
|*************************************************************************/

#ifndef FALSE
#define FALSE           0
#endif
#ifndef TRUE
#define TRUE            1
#endif

#ifndef CHAR
#define CHAR    unsigned char
#endif
#ifndef BYTE
#define BYTE    unsigned char
#endif
#ifndef WORD
#define WORD    unsigned short
#endif
#ifndef DWORD
#define DWORD   unsigned long
#endif
#ifndef BOOL
#define BOOL    int
#endif


typedef struct {
  BOOL    connected;
  BOOL    asynConnection;
  int     lastError;      /* store last error until reopen      */
  int     fd;
  DWORD   asynBaudrate;
  DWORD   repeatCount;
  BYTE    frameType;
  BYTE    frameCount;
  BYTE    bdiEvents;
  BYTE    txFrame[BDI_MAX_FRAME_SIZE];
  BYTE    rxFrame[BDI_MAX_FRAME_SIZE];

  void*   hCompress;
  int     loadError;      /* the first detected error during load     */
  BYTE    compressType;   /* the type of compression                  */
  DWORD   loadErrorAddr;  /* the last received error address          */
  DWORD   reqSeqNumber;   /* the next transmit sequence number        */
  DWORD   ackSeqNumber;   /* the first unacknowledged sequence number */
  BYTE    downloadFrame[LOAD_FRAME_COUNT][BDI_MAX_FRAME_SIZE];
  int     downloadCount[LOAD_FRAME_COUNT];
  BYTE    loadWindow;     /* the max.number of outstanding ack.frames */
  WORD    loadRepeatTime; /* the waiting time ack.frames              */
  WORD    loadMaxDataSize;/* the maximal size of the data part        */
} BDI_ChannelT;


/*************************************************************************
|  FUNCTIONS
|*************************************************************************/

int   BDI_Open(BDI_ChannelT* pCh, const char *host, DWORD param);
int   BDI_Close(BDI_ChannelT* pCh);

int   BDI_Transaction(      BDI_ChannelT* pCh,
                            int           commandLength,
                      const void*         commandData,
                            int           answerSize,
                            void*         answerData,
                            DWORD         commandTime);

BYTE  BDI_LnkGetPendingEvents(BDI_ChannelT* pCh);
int   BDI_LnkInitDownload(BDI_ChannelT* pCh, BYTE verifyMode, BOOL compress);
int   BDI_LnkDownload(BDI_ChannelT* pCh, DWORD address, WORD count, BYTE *data);
int   BDI_LnkEndDownload(BDI_ChannelT* pCh, DWORD *errorAddr);

#ifdef __cplusplus
}
#endif

#endif

