/*************************************************************************
|  COPYRIGHT (c) 2000 BY ABATRON AG
|*************************************************************************
|
|  PROJECT NAME: Simple bdiAccess demo
|  FILENAME    : myapp.c
|
|  COMPILER    : GCC
|
|  TARGET OS   : Linux
|  TARGET HW   : PC
|
|  PROGRAMMER  : Abatron / RD
|  CREATION    : 22.07.2005
|
|*************************************************************************
|
|  DESCRIPTION :
|  A simple command line application that uses BDI debug functions.
|
|*************************************************************************/

/*************************************************************************
|  INCLUDES
|*************************************************************************/

#include <ctype.h>
#include <unistd.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <sys/times.h>

#include "bdierror.h"
#include "bdicmd.h"
#include "bdilnkmt.h"
#include "bdiifcmt.h"


/*************************************************************************
|  DEFINES
|*************************************************************************/

#define TARGET_OKAY   0


/*************************************************************************
|  MACROS
|*************************************************************************/

/*************************************************************************
|  TYPEDEFS
|*************************************************************************/

/*************************************************************************
|  LOCALS
|*************************************************************************/

static DWORD      nextTargetAddr;  /* used for continous memory display */
static BOOL       terminate;
static BOOL       modeJTAG;
static void*      hBDI;


/****************************************************************************
 ****************************************************************************

    BDI_Append___ :

    Host independent helper function to append a numeric value to a buffer.
    The bytes will be stored in the link format order (Motorola byte order).

     INPUT  : value     value
              buffer    pointer to buffer
     OUTPUT : RETURN    pointer to next byte after the stored value

 ****************************************************************************/

static BYTE* BDI_AppendChar(char value, BYTE* buffer)
{
  *buffer++ = (BYTE)value;
  return buffer;
} /* BDI_AppendCHAR */

static BYTE* BDI_AppendByte(BYTE  value, BYTE* buffer)
{
  *buffer++ = value;
  return buffer;
} /* BDI_AppendByte */

static BYTE* BDI_AppendWord(WORD value, BYTE* buffer)
{
  *buffer++ = (BYTE)(value>>8);
  *buffer++ = (BYTE)value;
  return buffer;
} /* BDI_AppendWord */

static BYTE* BDI_AppendLong(DWORD value, BYTE* buffer)
{
  *buffer++ = (BYTE)(value>>24);
  *buffer++ = (BYTE)(value>>16);
  *buffer++ = (BYTE)(value>>8);
  *buffer++ = (BYTE)value;
  return buffer;
} /* BDI_AppendLong */


/****************************************************************************
 ****************************************************************************

    BDI_Extract___ :

    Host independent helper function to extract a numeric value from a buffer.
    The value must be stored in link format (Motorola byte order).

     INPUT  : buffer    pointer to buffer
     OUTPUT : value     pointer to variable where to store the value
              RETURN    pointer to next byte after the extracted value

 ****************************************************************************/

static BYTE* BDI_ExtractChar(char* value, BYTE* buffer)
{
  *value = (char)*buffer++;
  return buffer;
} /* BDI_ExtractCHAR */

static BYTE* BDI_ExtractByte(BYTE* value, BYTE* buffer)
{
  *value = *buffer++;
  return buffer;
} /* BDI_ExtractByte */

static BYTE* BDI_ExtractWord(WORD* value, BYTE* buffer)
{
  WORD x;
  x = (WORD)*buffer++;
  x = (x<<8) + (WORD)*buffer++;
  *value = x;
  return buffer;
} /* BDI_ExtractWord */

static BYTE* BDI_ExtractLong(DWORD* value, BYTE* buffer)
{
  DWORD x;
  x = (DWORD)*buffer++;
  x = (x<<8) + (DWORD)*buffer++;
  x = (x<<8) + (DWORD)*buffer++;
  x = (x<<8) + (DWORD)*buffer++;
  *value = x;
  return buffer;
} /* BDI_ExtractLong */


/****************************************************************************
 ****************************************************************************

    ValidEndOfLine:

    Test for a valid enf of line

    INPUT  : linePtr    pointer to the next character to read
    OUTPUT : RETURN     TRUE if valid end of line

 ****************************************************************************/

static BOOL ValidEndOfLine (const char* linePtr)
{
  while (*linePtr == ' ') linePtr++;
  return (*linePtr == 0);
} /* ValidEndOfLine */


/****************************************************************************
 ****************************************************************************

    ExtractNumber

    Extract a number (0xnnn for hex). Conversion stops if illegal character.

    INPUT  : getPtr     pointer to the next character to read
    OUTPUT : value      the extracted integer
             RETURN     pointer to the next unread character

 ****************************************************************************/

static char* ExtractHex (DWORD* value,  char* getPtr)
{
  DWORD num;
  DWORD digit;
  BYTE  c;

  while (*getPtr == ' ') getPtr++;
  num = 0;
  for (;;) {
    c = *getPtr;
    if      ((c >= '0') && (c <= '9')) digit = (DWORD)(c - '0');
    else if ((c >= 'A') && (c <= 'F')) digit = (DWORD)(c - 'A' + 10);
    else if ((c >= 'a') && (c <= 'f')) digit = (DWORD)(c - 'a' + 10);
    else break;
    num <<= 4;
    num += digit;
    getPtr++;
  } /* for */
  *value = num;
  return getPtr;
} /* ExtractHex */

static char* ExtractDecimal (DWORD* value,  char* getPtr)
{
  DWORD num;
  DWORD digit;
  BYTE  c;

  while (*getPtr == ' ') getPtr++;
  num = 0;
  for (;;) {
    c = *getPtr;
    if      ((c >= '0') && (c <= '9')) digit = (DWORD)(c - '0');
    else break;
    num *= 10;
    num += digit;
    getPtr++;
  } /* for */
  *value = num;
  return getPtr;
} /* ExtractDecimal */


static char* ExtractNumber (DWORD* value,  char* getPtr)
{
  while (*getPtr == ' ') getPtr++;
  if ((*getPtr == '0') && ((*(getPtr+1) == 'x') || (*(getPtr+1) == 'X'))) {
    getPtr +=2;
    return ExtractHex(value, getPtr);
  } /* if */
  else {
    return ExtractDecimal(value, getPtr);
  } /* else */
} /* ExtractNumber */


/****************************************************************************
 ****************************************************************************

    ExtractString

    Extract a string until the next seperator.

    INPUT  : getPtr       pointer to the next character to read
    OUTPUT : string       a pointer to the first character of the string
             RETURN       pointer to the next unread character after the enumaration

 ****************************************************************************/

char* ExtractString (char* string, char* getPtr)
{

  /* skip spaces */
  while (*getPtr == ' ') getPtr++;

  /* copy string */
  while ((*getPtr != ' ') && (*getPtr != ';') && (*getPtr != 0)) {
    *string++ = *getPtr++;
  } /* while */

  *string = 0;
  return getPtr;
} /* ExtractString */


/****************************************************************************
 ****************************************************************************

    MatchName

    Searches for a name in a name table. The search is not case sensitive.
    All transmitted characters until the next special character are compared.
    At least one name character must be present.

    INPUT  : tableSize  the number of enties in the name table
             nameTable  the name table to search
             getPtr     pointer to the command frame
    OUTPUT : index      the index if a match is found or -1 if no match
             RETURN     pointer to the next unread character in the command frame

 ****************************************************************************/

static char* MatchName(int tableSize, char** nameTable, int* index, char* getPtr)
{
  int   i;
  char  firstChar;
  char  c;
  char* name;
  char* reference;

  firstChar = *getPtr;
  if (firstChar >= 'a') firstChar -= ('a'-'A'); /* make upper */
  for (i=0; i < tableSize; i++) {
    reference = *nameTable;
    if (*reference++ == firstChar) {
      name = getPtr;
      do {
        name++;
        c = *name;

        /* valid end */
        if (c <= ' ') {
          *index = i;
          return name;
        } /* if */

        if (c >= 'a') c -= ('a'-'A'); /* make upper */

        /* check invalid char */
        if (!(c >= 'A' && c <= 'Z') && !(c >= '0' && c <= '9')) {
          *index = -1;
          return name;
        } /* if */

      } while (*reference++ == c);

    } /* if */
    nameTable++;
  } /* for */

  /* no match found */
  *index = -1;
  return getPtr;
} /* MatchName */


/****************************************************************************
 ****************************************************************************

    DisplayHelp :

    Display the help screen.

    INPUT  : -
    OUTPUT : -

 ****************************************************************************/

static char* helpScreen[] = {
"MD    [<address>] [<count>]  display target memory as word (32bit)",
"MDH   [<address>] [<count>]  display target memory as half word (16bit)",
"MDB   [<address>] [<count>]  display target memory as byte (8bit)",
"MM    <address>   <value>    modify word (32bit) to target memory",
"MMH   <address>   <value>    modify half word (16bit) to target memory",
"MMB   <address>   <value>    modify byte (8bit) to target memory",
"RESET                        reset the target",
"INFO                         display information about the target",
"HELP                         display command list",
"EXIT                         terminate"
};

static void DisplayHelp(void)
{
  int   i;
  char  line[80];

  for (i=0; i < (int)(sizeof helpScreen / sizeof helpScreen[0]); i++) {
    printf("%s\n", helpScreen[i]);
    if ((i % 20) == 19) {
      printf("press enter to continue...");
      gets(line);
    } /* if */
  } /* for */
} /* DisplayHelp */


/****************************************************************************
 ****************************************************************************

    DisplayLong :

    Display memory words (32bit).

    INPUT  : address    the start address
             count      the number of word to display
    OUTPUT : RETURN     error code

 ****************************************************************************/

static int DisplayLong(DWORD address, WORD count)
{
  int   i;
  int   j;
  int   result;
  DWORD mem[4];
  BYTE  buffer[4];

  /* display memory as block */
  result = TARGET_OKAY;
  if (count == 0) {
    for (i=0; i<16; i++) {
      for (j=0; j<4; j++) {
        if (result == TARGET_OKAY) result = BDI_MemoryRead(hBDI,
                                                           address + 4 * j,
                                                           BDI_AT_DEFAULT,
                                                           4,
                                                           buffer);
        (void)BDI_ExtractLong(&mem[j], buffer);
      } /* for */
      if (result == TARGET_OKAY) {
        (void)printf("%08lx : %08lx %08lx %08lx %08lx\n", address, mem[0], mem[1], mem[2], mem[3]);
        address += 16;
      } /* if */
      else {
        break;
      } /* else */
    } /* for */
  } /* if */

  /* display exact the requested values */
  while ((result == TARGET_OKAY && count--)) {
    result = BDI_MemoryRead(hBDI, address, BDI_AT_DEFAULT, 4, buffer);
    (void)BDI_ExtractLong(&mem[0], buffer);
    if (result == TARGET_OKAY) {
      (void)printf("%08lx : 0x%08lx  % 11li\n", address, mem[0], mem[0]);
      address += 4;
    } /* if */
  } /* while */

  nextTargetAddr = address;
  return result;
} /* DisplayLong */


/****************************************************************************
 ****************************************************************************

    DisplayWord :

    Display memory half words (16bit).

    INPUT  : address    the start address
             count      the number of word to display
    OUTPUT : RETURN     error code

 ****************************************************************************/

static int DisplayWord(DWORD address, WORD count)
{
  int   i;
  int   j;
  int   result;
  WORD  mem[8];
  BYTE  buffer[2];

  /* display memory as block */
  result = TARGET_OKAY;
  if (count == 0) {
    for (i=0; i<16; i++) {
      for (j=0; j<8; j++) {
        if (result == TARGET_OKAY) result = BDI_MemoryRead(hBDI,
                                                           address + 2 * j,
                                                           BDI_AT_DEFAULT,
                                                           2,
                                                           buffer);
        (void)BDI_ExtractWord(&mem[j], buffer);
      } /* for */
      if (result == TARGET_OKAY) {
        printf("%08lx : %04x %04x %04x %04x %04x %04x %04x %04x\n",
               address, mem[0],mem[1],mem[2],mem[3],mem[4],mem[5],mem[6],mem[7]);
        address += 16;
      } /* if */
      else {
        break;
      } /* else */
    } /* for */
  } /* if */

  /* display exact the requuested values */
  while ((result == TARGET_OKAY && count--)) {
    result = BDI_MemoryRead(hBDI, address, BDI_AT_DEFAULT, 2, buffer);
    (void)BDI_ExtractWord(&mem[0], buffer);
    if (result == TARGET_OKAY) {
      printf("%08lx : 0x%04x  % 6i\n", address, mem[0], mem[0]);
      address += 2;
    } /* if */
  } /* while */

  nextTargetAddr = address;
  return result;
} /* DisplayWord */


/****************************************************************************
 ****************************************************************************

    DisplayByte :

    Display memory as bytes (8bit).

    INPUT  : address    the start address
             count      the number of word to display
    OUTPUT : RETURN     error code

 ****************************************************************************/

static int DisplayByte(DWORD address, WORD count)
{
  int   i;
  int   j;
  int   result;
  BYTE  mem[8];

  /* display memory as block */
  result = TARGET_OKAY;
  if (count == 0) {
    for (i=0; i<16; i++) {
      for (j=0; j<8; j++) {
        if (result == TARGET_OKAY) result = BDI_MemoryRead(hBDI,
                                                           address + j,
                                                           BDI_AT_DEFAULT,
                                                           1,
                                                           &mem[j]);
      } /* for */
      if (result == TARGET_OKAY) {
        printf("%08lx : %02x %02x %02x %02x %02x %02x %02x %02x\n",
               address, mem[0],mem[1],mem[2],mem[3],mem[4],mem[5],mem[6],mem[7]);
        address += 8;
      } /* if */
      else {
        break;
      } /* else */
    } /* for */
  } /* if */

  /* display exact the requuested values */
  while ((result == TARGET_OKAY && count--)) {
    result = BDI_MemoryRead(hBDI, address, BDI_AT_DEFAULT, 1, &mem[0]);
    if (result == TARGET_OKAY) {
      printf("%08lx : 0x%02x  % 4i\n", address, mem[0], mem[0]);
      address++;
    } /* if */
  } /* while */

  nextTargetAddr = address;
  return result;
} /* DisplayByte */


/****************************************************************************
 ****************************************************************************

    DisplayTargetInfo :

    Display the target information.

    INPUT  : -
    OUTPUT : -

 ****************************************************************************/

static int DisplayTargetInfo(void)
{
  int       result;
  WORD      state;
  DWORD     timeStamp;
  WORD      infoCount;
  DWORD     infoValue[10];
  int       errorCode;
  WORD      i;

  infoCount = 10;
  result = BDI_StateGet(hBDI, &state, &errorCode, &timeStamp, &infoCount, infoValue);
  if (result != TARGET_OKAY) return result;

  switch (state) {

    case 0:
      printf("BDI state         : reset\n");
      break;

    case 1:
      printf("Target state      : debug mode\n");
      break;

    case 2:
      printf("BDI state         : running\n");
      break;

    case 3:
      printf("BDI state         : error\n");
      printf("Error info        : %s\n", BDI_ErrorTextGet(errorCode));
      break;

    default:
      printf("BDI state         : unknown\n");
      break;

  } /* switch */

  printf("Reset Timestamp   : %lu\n", timeStamp);

  for (i = 0; i < infoCount; i++) {
    printf("Info[%i]           : 0x%08lx\n", i+1, infoValue[i]);
  } /* while */

  return TARGET_OKAY;
} /* DisplayTargetInfo */


/****************************************************************************
 ****************************************************************************

  LoadBinary:

  Load an binary file.

  INPUT  : fileName     the file name on the host
           verifyMode   the verify mode to use
           startAddr    points to the address where to load the file
  OUTPUT : errorAddr    the address of the failing memory
           RETURN       error code

 ****************************************************************************/

int LoadBinary(char* fileName, int verifyMode, DWORD startAddr, DWORD* errorAddr)
{
  FILE*   file;
  int     loadResult;
  int     rxCount;
  BYTE    rxBlock[1024];
  DWORD   loadAddr;


  /* open binary file */
  file = fopen(fileName,"rb");
  if (file == NULL) return BDI_ERR_FILE_ACCESS;

  /* init download */
  loadResult = BDI_InitDownload(hBDI, (BYTE)verifyMode, TRUE);

  /* download file */
  loadAddr = startAddr;
  do {
    rxCount = fread(rxBlock, 1, sizeof rxBlock, file);
    if (rxCount > 0) {
      loadResult = BDI_Download(hBDI, loadAddr, (WORD)rxCount, rxBlock);
      loadAddr  += rxCount;
    } /* if */
  } while ((rxCount > 0) && (loadResult == BDI_OKAY));

  /* terminate download */
  if (loadResult == BDI_OKAY) {
    loadResult = BDI_EndDownload(hBDI, errorAddr);
  } /* if */
  else {
    (void)BDI_EndDownload(hBDI, errorAddr);
  } /* else */

  fclose(file);
  return loadResult;
} /* LoadBinary */


/****************************************************************************
 ****************************************************************************

    Program5554 :

 ****************************************************************************/

static BDI_InitTypeT initList5554[] = {
  {BDI_INIT_P55_WSPR, 8007,       100},        // Wake-up delay 100 ms
  {BDI_INIT_P55_WSPR, 8004,       1},          // Select memroy access via Nexus

  {BDI_INIT_P55_WTLB, 0xFFF0058A, 0xFFF0003F}, // Bridge B: 0xfff00000 -> 0xfff00000,   1MB, --I-G-
  {BDI_INIT_P55_WTLB, 0xC3F0058A, 0xC3F0003F}, // Bridge A: 0xc3f00000 -> 0xc3f00000,   1MB, --I-G-
  {BDI_INIT_P55_WTLB, 0x4000048A, 0x4000003F}, // Int.SRAM: 0x40000000 -> 0x40000000, 256KB, --I-G-
  {BDI_INIT_P55_WTLB, 0x0000078A, 0x0000003F}, // Flash   : 0x00000000 -> 0x00000000,  16MB, --I-G-
  {BDI_INIT_P55_FILL, 0x40000000, 0x10000   }, // init ECC-SRAM
  {BDI_INIT_P55_WM32, 0xC3F80000, 0x01000000}, // FMPLL_SYNCR: MFD=2,RFD=0 -> fsys = 48MHz

   // Lock/Unlock Flash Blocks for Erase/Programming
  {BDI_INIT_P55_WM32, 0xC3F88004, 0xA1A11111}, //  FLASH_LMLR : unlock register
  {BDI_INIT_P55_WM32, 0xC3F88004, 0x001CFFFF}, // FLASH_LMLR : lock/unlock blocks
  {BDI_INIT_P55_WM32, 0xC3F8800C, 0xC3C33333}, // FLASH_SLMLR: unlock register
  {BDI_INIT_P55_WM32, 0xC3F8800C, 0x001CFFFF}, // FLASH_SLMLR: lock/unlock blocks
  {BDI_INIT_P55_WM32, 0xC3F88008, 0xB2B22222}, // FLASH_HLR  : unlock register
  {BDI_INIT_P55_WM32, 0xC3F88008, 0x0FFFFFF0}, // FLASH_HLR  : lock/unlock blocks
};



static int Program5554(void)
{
  int   result;
  DWORD errorAddr;

  /* reset and init target */
  result = BDI_TargetStartup(hBDI,
                             100,         // 100ms reset time
                             (0<<8) | 3,  // MPC5554, 8 MHz JTAG clock
                             sizeof initList5554 / sizeof initList5554[0],
                             initList5554);

  /* setup flash programming */
  result = BDI_FlashSetType(hBDI,
                            BDI_FLASH_H7F,
                            0x80000,
                            32,
                            32,
                            0x40001000);
  if (result != BDI_OKAY) return result;


  /* erase flash  */
  printf("erasing ...\n");
  if (result == BDI_OKAY) result = BDI_FlashErase(hBDI, BDI_ERASE_MID,  0x00000003);
  if (result == BDI_OKAY) result = BDI_FlashErase(hBDI, BDI_ERASE_HIGH, 0x0000000F);
  printf((result == BDI_OKAY) ? "passed\n" : "failed\n");
  if (result != BDI_OKAY) return result;

  printf("programming...\n");
  result = BDI_FlashWriteBinary(hBDI, "E:/temp/dump512k.bin", 0x00040000, &errorAddr);
  printf((result == BDI_OKAY) ? "passed\n" : "failed\n");
  if (result != BDI_OKAY) return result;

  printf("verifing...\n");
  result = BDI_VerifyBinary(hBDI, "E:/temp/dump512k.bin", 0x00040000, &errorAddr);
  printf("passed\n");

  return result;
} /* Program5554 */


/****************************************************************************
 ****************************************************************************

    ProcessCommand :

    Process a command line

    INPUT  : line       the command line
    OUTPUT : -

 ****************************************************************************/


static void ProcessCommand(char* line)
{
  static char* nameTable[] = {
     "MD","MDH","MDB","MM","MMH","MMB",
     "RESET","INFO","LOAD","VERIFY","PROG5554","JTAG","HELP","EXIT"
  };

  typedef enum {
    N_MD,N_MDH,N_MDB,N_MM,N_MMH,N_MMB,
    N_RESET,N_INFO,N_LOAD,N_VERIFY,N_PROG5554,N_JTAG,N_HELP,N_EXIT
  } NameT;

  char*         linePtr;
  int           command;
  int           paramCount;
  DWORD         param1;
  DWORD         param2;
  DWORD         param3;
  DWORD         errorParam;
  int           result;
  BYTE          buffer[4];
  char          fileName[80];

  /* extract command and parameters */
  paramCount = 0;
  param1     = 0;
  param2     = 0;
  param3     = 0;

  /* get command */
  linePtr = MatchName(sizeof nameTable / sizeof nameTable[0],
                      nameTable,
                      &command,
                      line);

  /* get parameters */
  if (!ValidEndOfLine(linePtr)) {
    linePtr = ExtractNumber(&param1, linePtr);
    paramCount++;
  } /* if */
  if (!ValidEndOfLine(linePtr)) {
    if ((command == N_LOAD) || (command == N_VERIFY))
      linePtr = ExtractString(fileName, linePtr);
    else
      linePtr = ExtractNumber(&param2, linePtr);
    paramCount++;
  } /* if */
  if (!ValidEndOfLine(linePtr)) {
    linePtr = ExtractNumber(&param3, linePtr);
    paramCount++;
  } /* if */

  if ((command >= 0) && (ValidEndOfLine(linePtr))) {
    result = TARGET_OKAY;
    switch (command) {

      case N_MD:
        if (paramCount == 0) param1 = nextTargetAddr;
        result = DisplayLong(param1, (WORD)param2);
        break;

      case N_MDH:
        if (paramCount == 0) param1 = nextTargetAddr;
        result = DisplayWord(param1, (WORD)param2);
        break;

      case N_MDB:
        if (paramCount == 0) param1 = nextTargetAddr;
        result = DisplayByte(param1, (WORD)param2);
        break;

      case N_MM:
        if (paramCount == 2) {
          (void)BDI_AppendLong(param2, buffer);
          result = BDI_MemoryWrite(hBDI, param1, BDI_AT_DEFAULT, 4, buffer);
        } /* if */
        else
          printf("# syntax is MM <address> <value>\n");
        break;

      case N_MMH:
        if (paramCount == 2) {
          (void)BDI_AppendWord((WORD)param2, buffer);
          result = BDI_MemoryWrite(hBDI, param1, BDI_AT_DEFAULT, 2, buffer);
        } /* if */
        else
          printf("# syntax is MMH <address> <value>\n");
        break;

      case N_MMB:
        if (paramCount == 2) {
          (void)BDI_AppendByte((BYTE)param2, buffer);
          result = BDI_MemoryWrite(hBDI, param1, BDI_AT_DEFAULT, 1, buffer);
        } /* if */
        else
          printf("# syntax is MMB <address> <value>\n");
        break;

      case N_RESET:
        result = BDI_TargetReset(hBDI);
        break;

      case N_INFO:
        result = DisplayTargetInfo();
        break;

      case N_LOAD:
        if ((paramCount == 2) && (fileName[0] != 0)) {
          printf("Loading %s , please wait ....\n", fileName);
          result = LoadBinary(fileName, BDI_NO_VERIFY, param1, &errorParam);
          if (result == TARGET_OKAY) {
            printf("Loading program file passed\n");
          } /* if */
          else {
            printf("Download faild at 0x%08lx\n", errorParam);
          } /* else */
        } /* if */
        else
          printf("# syntax is LOAD <addr> <filename>");
        break;

      case N_VERIFY:
        if ((paramCount == 2) && (fileName[0] != 0)) {
          printf("Verifying %s , please wait ....\n", fileName);
          result = LoadBinary(fileName, BDI_ONLY_VERIFY, param1, &errorParam);
          if (result == TARGET_OKAY) {
            printf("Verifying target memory passed\n");
          } /* if */
          else {
            printf("Verify faild at 0x%08lx\n", errorParam);
          } /* else */
        } /* if */
        else
          printf("# syntax is VERIFY <addr> <filename>\n");
        break;

      case N_PROG5554:
        result = Program5554();
        break;

      case N_JTAG:
        result = BDI_DoJtag(hBDI, JTAG_CMD_OPEN, 0, 0, NULL, NULL);
        if (result == TARGET_OKAY) modeJTAG = TRUE;
        break;

      case N_HELP:
        DisplayHelp();
        break;

      case N_EXIT:
        terminate = TRUE;
        break;

      default:
        printf("Sorry, command not jet implemented!\n");
        break;

    } /* switch */

    /* display error */
    if (result != TARGET_OKAY) {
      printf("%s\n", BDI_ErrorTextGet(result));
    } /* if */
  } /* if */

  else {
    printf("# syntax error in command\n");
  } /* else */

} /* ProcessCommand */


/****************************************************************************
 ****************************************************************************

    TELNET_ProcessJTAG :

    Process a JTAG command line

    INPUT  : line       the command line
    OUTPUT : -

 ****************************************************************************/

#define JTAG_DATA_BYTES         32
#define JTAG_DATA_BITS          (8 * JTAG_DATA_BYTES)

/* Binary to ASCII-HEX conversion table */
static const char BinToHex[] = {'0','1','2','3','4','5','6','7','8','9',
                                'a','b','c','d','e','f'};

static char* jtagHelp[] = {
"TRST  {0|1}                  assert (1) or release (0) TRST",
"RST   {0|1}                  assert (1) or release (0) RST",
"HALT  {0|1}                  assert (1) or release (0) DINT",
"SPEED <n>                    select JTAG clock frequency",
"CLK   <count> <tms>          clock TAP with requested TMS value",
"SCAN  <ctrl><len><...b2b1b0> scan with provided parameters",
"RIR   <len>                  read  IR, zero is scanned in",
"RDR   <len>                  read  DR, zero is scanned in",
"WIR   <len> <...b2b1b0>      write IR, b0 is first scanned",
"WDR   <len> <...b2b1b0>      write DR, b0 is first scanned",
"XIR   <len> <...b2b1b0>      xchg  IR, b0 is first scanned",
"XDR   <len> <...b2b1b0>      xchg  DR, b0 is first scanned",
"                             len : the number of bits 1..256",
"                             bx  : a data byte, two hex digits",
"HELP                         display JTAG command list",
"EXIT                         terminate JTAG mode"
};

static void DisplayJtagHelp(void)
{
  int   i;
  char  line[80];

  for (i=0; i < (int)(sizeof jtagHelp / sizeof jtagHelp[0]); i++) {
    printf("%s\n", jtagHelp[i]);
  } /* for */
} /* DisplayJtagHelp */


static const char* ExtractHexByte(BYTE* value, const char* getPtr)
{
  BYTE  num;
  BYTE  digit;
  BYTE  c;
  int   i;

  while (*getPtr == ' ') getPtr++;
  num = 0;
  for (i = 0; i < 2; i++) {
    c = *getPtr;
    if      ((c >= '0') && (c <= '9')) digit = (BYTE)(c - '0');
    else if ((c >= 'a') && (c <= 'f')) digit = (BYTE)(c - 'a' + 10);
    else if ((c >= 'A') && (c <= 'F')) digit = (BYTE)(c - 'A' + 10);
    else break;
    num <<= 4;
    num += digit;
    getPtr++;
  } /* for */
  *value = num;
  return getPtr;
} /* ExtractHexByte */


static void ProcessJTAG(char* line)
{
  static char* jtagTable[] = {
     "TRST","RST","HALT","CLK","SPEED","SCAN",
     "RIR","RDR","WIR","WDR","XIR","XDR","HELP","EXIT"
  };

  typedef enum {
    N_TRST,N_RST,N_HALT,N_CLK,N_SPEED,N_SCAN,
    N_RIR,N_RDR,N_WIR,N_WDR,N_XIR,N_XDR,N_HELP,N_EXIT
  } JtagT;

  const char*   linePtr;
  int           result;
  int           command;
  int           paramCount;
  DWORD         param1;
  DWORD         param2;
  DWORD         index;
  BYTE          dataWR[JTAG_DATA_BYTES];
  BYTE          dataRD[JTAG_DATA_BYTES];
  char          szData[2 * JTAG_DATA_BYTES + 1];
  char*         putPtr;

  /* extract command and parameters */
  paramCount = 0;
  param1     = 0;
  param2     = 0;
  (void)memset(dataRD, 0, sizeof dataRD);
  (void)memset(dataWR, 0, sizeof dataWR);
  linePtr = MatchName(sizeof jtagTable / sizeof jtagTable[0],
                      jtagTable,
                      &command,
                      line);

  /* get 1. parameter */
  if (!ValidEndOfLine(linePtr)) {
    linePtr = ExtractNumber(&param1, (char*)linePtr);
    paramCount++;
    if (   (command == N_WIR) || (command == N_WDR)
        || (command == N_XIR) || (command == N_XDR)) {
      if (param1 > JTAG_DATA_BITS) {
        printf("# maximum data length is 256 bits\n");
        return;
      } /* if */
      if (param1 == 0) {
        printf("# minimal data length is 1 bit\n");
        return;
      } /* if */
    } /* if */
  } /* if */

  /* get 2. parameter */
  if (!ValidEndOfLine(linePtr)) {
    if ((command == N_WIR) || (command == N_WDR) || (command == N_XIR) || (command == N_XDR)) {
      while (*linePtr == ' ') linePtr++;
      index = (param1 + 7) / 8;
      if (strlen(linePtr) != (2 * index)) {
        printf("# data entry does not match length\n");
        return;
      } /* if */
      while (index > 0) {
        index--;
        linePtr = ExtractHexByte(&dataWR[index], linePtr);
      } /* while */
    } /* if */
    else {
      linePtr = ExtractNumber(&param2, (char*)linePtr);
    } /* else */
    paramCount++;
    if (command == N_SCAN) {
      if (param2 > JTAG_DATA_BITS) {
        printf("# maximum data length is 256 bits\n");
        return;
      } /* if */
      if (param2 == 0) {
        printf("# minimal data length is 1 bit\n");
        return;
      } /* if */
    } /* if */
  } /* if */

  /* get 3. parameter */
  if (!ValidEndOfLine(linePtr)) {
    if (command == N_SCAN) {
      while (*linePtr == ' ') linePtr++;
      index = (param2 + 7) / 8;
      if (strlen(linePtr) != (2 * index)) {
        printf("# data entry does not match length\n");
        return;
      } /* if */
      while (index > 0) {
        index--;
        linePtr = ExtractHexByte(&dataWR[index], linePtr);
      } /* while */
    } /* if */
    paramCount++;
  } /* if */

  /* process command */
  result = TARGET_OKAY;
  if ((command >= 0) && (ValidEndOfLine(linePtr))) {
    switch (command) {

      case N_TRST:
        result = BDI_DoJtag(hBDI,
                            JTAG_CMD_PORT,
                            (BYTE)(((param1) ? JTAG_PORT_LOW : JTAG_PORT_HIGH) << 0),
                            0,
                            NULL,
                            NULL);
        break;

      case N_RST:
        result = BDI_DoJtag(hBDI,
                            JTAG_CMD_PORT,
                            (BYTE)(((param1) ? JTAG_PORT_LOW : JTAG_PORT_HIGH) << 2),
                            0,
                            NULL,
                            NULL);
        break;

      case N_HALT:
        result = BDI_DoJtag(hBDI,
                            JTAG_CMD_PORT,
                            (BYTE)(((param1) ? JTAG_PORT_LOW : JTAG_PORT_HIGH) << 4),
                            0,
                            NULL,
                            NULL);
        break;

      case N_CLK:
        if ((param1 >= 1) && (param1 <= 0xFFFF)) {
          result = BDI_DoJtag(hBDI,
                              JTAG_CMD_CLOCK,
                              (BYTE)((param2) ? JTAG_CLOCK_TMS : 0),
                              (WORD)param1,
                              NULL,
                              NULL);
        } /* if */
        else {
          printf("# valid count is 1...64K\n");
        } /* else */
        break;

      case N_SPEED:
        if (param1 <= 6) {
          result = BDI_DoJtag(hBDI,
                              JTAG_CMD_SPEED,
                              (BYTE)param1,
                              0,
                              NULL,
                              NULL);
        } /* if */
        else {
          printf("# valid speed is 0...6\n");
        } /* else */
        break;

      case N_SCAN:
        result = BDI_DoJtag(hBDI,
                            JTAG_CMD_SCAN,
                            (BYTE)param1,
                            (WORD)param2,
                            dataWR,
                            dataRD);
        break;

      case N_RIR:
      case N_RDR:
        result = BDI_DoJtag(hBDI, JTAG_CMD_CLOCK, 0, 1, NULL, NULL);
        result = BDI_DoJtag(hBDI,
                            (BYTE)((command == N_RIR) ? JTAG_CMD_IRSCAN : JTAG_CMD_DRSCAN),
                            0,
                            (WORD)param1,
                            dataWR,
                            dataRD);
        break;

      case N_WIR:
      case N_WDR:
        result = BDI_DoJtag(hBDI, JTAG_CMD_CLOCK, 0, 1, NULL, NULL);
        result = BDI_DoJtag(hBDI,
                            (BYTE)((command == N_WIR) ? JTAG_CMD_IRSCAN : JTAG_CMD_DRSCAN),
                            0,
                            (WORD)param1,
                            dataWR,
                            NULL);
        break;

      case N_XIR:
      case N_XDR:
        result = BDI_DoJtag(hBDI, JTAG_CMD_CLOCK, 0, 1, NULL, NULL);
        result = BDI_DoJtag(hBDI,
                            (BYTE)((command == N_XIR) ? JTAG_CMD_IRSCAN : JTAG_CMD_DRSCAN),
                            0,
                            (WORD)param1,
                            dataWR,
                            dataRD);
        break;

      case N_HELP:
        DisplayJtagHelp();
        break;

      case N_EXIT:
        result = BDI_DoJtag(hBDI, JTAG_CMD_CLOSE, 0, 0, NULL, NULL);
        modeJTAG = FALSE;
        break;

      default:
        printf("Sorry, command not jet implemented!\n");
        break;

    } /* switch */

    /* display error */
    if (result != TARGET_OKAY) {
      printf("%s\n", BDI_ErrorTextGet(result));
    } /* if */

    /* display data */
    if (    (command == N_RIR) || (command == N_RDR)
         || (command == N_XIR) || (command == N_XDR)
         || (command == N_SCAN)) {
      index = (command == N_SCAN) ? param2 : param1;
      index = (index + 7) / 8;
      putPtr = szData;
      while (index > 0) {
        index--;
        *putPtr++ = BinToHex[(dataRD[index] >> 4) & 0x0F];
        *putPtr++ = BinToHex[dataRD[index] & 0x0F];
      } /* while */
      *putPtr = 0;
      printf("%s\n", szData);
    } /* if */

  } /* if */
  else {
    printf("# invalid command in JTAG mode\n");
    printf("# enter exit to terminate JTAG mode\n");
  } /* else */

} /* ProcessJTAG */


/*************************************************************************
|  MAIN
|*************************************************************************/

int main( int argc, char *argv[ ])
{
  char  rxLine[128];
  char  host[80];
  int   param;
  int   result;
  WORD  bdiType;
  WORD  fwType;
  WORD  fwVersion;

  /* get a BDI handle */
  hBDI = BDI_CreateHandle();
  if (hBDI == NULL) {
    printf("Getting a BDI handle failed\n");
    printf("press enter to continue...");
    gets(rxLine);
    exit(0);
  } /* if */

  /* connect to BDI */
  printf("Connecting to BDI, please wait...\r");
  if (argc == 3) {
    sprintf(host, "%s", argv[1]);
    param = atoi(argv[2]);
  } /* if */
  else  {
    sprintf(host, "151.120.25.101");
    param = 2001;
  } /* if */
  result = BDI_Connect(hBDI, host, param);
  if (result != 0) {
    BDI_CloseHandle(hBDI);
    printf("Connecting to BDI failed with error %i\n", result);
    printf("press enter to continue...");
    gets(rxLine);
    exit(0);
  } /* if */
  result = BDI_TypeGet(hBDI, &bdiType, &fwType, &fwVersion);
  printf("\n\n");
  printf("BDI: %i FW: %i / %i \n", bdiType, fwType, fwVersion);

  printf("\n\n");
  printf("Simple BDI Interface Demo\n");
  printf("=========================\n\n");
  printf("enter HELP for command list\n");

  terminate = FALSE;
  modeJTAG  = FALSE;
  do {
    printf((modeJTAG) ? "JTAG>" : "BDI>");
    if (gets(rxLine)) {
      if (modeJTAG) ProcessJTAG(rxLine);
      else          ProcessCommand(rxLine);
    } /* if */
  } while (!terminate);

  result = BDI_Disconnect(hBDI);
  BDI_CloseHandle(hBDI);

  return 0;
} /* main */
